// PathsInfoDialog.cpp : PathsInfoDialog class implementation
// Author: Ovidiu Cucu
// Homepage: https://codexpert.ro/blog/

#include "pch.h"
#include "framework.h"
#include "PathsInfo.h"
#include "PathsInfoDialog.h"

// Notes: 
// #include <PathCch.h> in framework.h header
// add Pathcch.lib to project linker dependencies

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CPathsInfoDialog::CPathsInfoDialog(CWnd* pParent /*=nullptr*/)
	: CDialogEx(IDD_PATHSINFO_DIALOG, pParent)
	, m_strUserSelectedFolder(_T(""))
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CPathsInfoDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_EDIT_CURRENT_DIRECTORY, m_strCurrentDirectory);
	DDX_Text(pDX, IDC_EDIT_EXE_FILE_PATH, m_strExeFilePath);
	DDX_Text(pDX, IDC_EDIT_ROAMING_APPDATA_FOLDER, m_strAppDataFolder);
	DDX_Text(pDX, IDC_EDIT_USER_SELECTED_FOLDER, m_strUserSelectedFolder);
}

BEGIN_MESSAGE_MAP(CPathsInfoDialog, CDialogEx)
	ON_BN_CLICKED(IDC_BUTTON_SELECT_FOLDER, &CPathsInfoDialog::OnButtonSelectFolder)
END_MESSAGE_MAP()

BOOL CPathsInfoDialog::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	SetDataToDialog();

	return TRUE;
}

void CPathsInfoDialog::SetDataToDialog()
{
	HRESULT hr = GetAppCurrentDirectory(m_strCurrentDirectory);
	if (FAILED(hr))
	{
		m_strCurrentDirectory.Format(_T("GetAppCurrentDirectory faled\nError: %08X"), hr);
	}

	hr = GetExeFilePath(m_strExeFilePath);
	if (FAILED(hr))
	{
		m_strCurrentDirectory.Format(_T("GetExeFilePath faled\nError: %08X"), hr);
	}

	hr = GetRoamingAppDataFolder(m_strAppDataFolder);
	if (FAILED(hr))
	{
		m_strCurrentDirectory.Format(_T("GetRoamingAppDataKnownFolder faled\nError: %08X"), hr);
	}

	UpdateData(FALSE);
}

HRESULT CPathsInfoDialog::GetAppCurrentDirectory(CString& strDir)
{
	// get required buffer length, including terminating null character
	DWORD dwLength = ::GetCurrentDirectory(0, NULL) + 1;

	// get the application/process current directory
	if (0 == ::GetCurrentDirectory(dwLength, strDir.GetBufferSetLength(dwLength)))
	{
		return  HRESULT_FROM_WIN32(::GetLastError());
	}
	strDir.ReleaseBuffer();
	return S_OK;
}

HRESULT CPathsInfoDialog::GetExeFilePath(CString& strPath)
{
	DWORD dwRet = NOERROR;

	// get executable module handle
	HMODULE hModule = ::GetModuleHandle(NULL);

	// get the executable full path and file name
	DWORD dwSize = MAX_PATH + 1;

	::SetLastError(NOERROR);
	::GetModuleFileName(hModule, strPath.GetBufferSetLength(dwSize), dwSize);
	strPath.ReleaseBuffer();

	HRESULT hr = HRESULT_FROM_WIN32(::GetLastError());
	if (FAILED(hr))
		return hr;

	size_t cchPath = strPath.GetLength() + 1;
	hr = ::PathCchRemoveFileSpec(strPath.GetBuffer(), cchPath);
	strPath.ReleaseBuffer();
	return hr;
}


HRESULT CPathsInfoDialog::GetRoamingAppDataFolder(CString& strPath)
{
	PWSTR path{ NULL };
	HRESULT hr = ::SHGetKnownFolderPath(FOLDERID_RoamingAppData, 0, NULL, &path);
	if (FAILED(hr))
		return hr;

	strPath = path;
	::CoTaskMemFree(path);
	return S_OK;
}

void CPathsInfoDialog::OnButtonSelectFolder()
{
	CFolderPickerDialog dlg;
	if (IDOK == dlg.DoModal())
	{
		m_strUserSelectedFolder = dlg.GetFolderPath();
		SetDataToDialog();
	}
}

void CPathsInfoDialog::OnOK()
{
	//	CDialogEx::OnOK(); // just prevent closing when hit 'Enter'
}

